@@ -1,20 +1,21 @@
 # -*- coding:utf-8 -*- 
 
 from BDMLtools.clearner import outliersTransformer,dtypeAllocator,nanTransformer,dtStandardization
-from BDMLtools.report import businessReport,EDAReport,varReport,varGroupsReport,varReportSinge
-from BDMLtools.selector import binSelector,binFreq,binPretty,binTree,binChi2,binKmeans
+from BDMLtools.report import businessReport,EDAReport,varReport,varGroupsReport,varReportSinge,varGroupsPlot
+from BDMLtools.selector import binSelector,binAdjuster,binFreq,binPretty,binTree,binChi2,binKmeans
 from BDMLtools.selector import faSelector
 from BDMLtools.selector import stepLogit,cardScorer
 from BDMLtools.selector import preSelector,corrSelector,prefitModel
 #from BDMLtools.selector import RFECVSelector
 from BDMLtools.selector import lassoSelector
 from BDMLtools.plotter import BaseWoePlotter
+from BDMLtools.plotter import perfEval
 from BDMLtools.encoder import woeTransformer
 from BDMLtools.tuner import girdTuner,hgirdTuner
 from BDMLtools.tuner import BayesianXGBTuner,BayesianLgbmTuner,shapCheck
 
 
-__version__ = '0.1.7'
+__version__ = '0.1.9'
 
 __all__ = (    
     dtStandardization,
@@ -27,7 +28,9 @@ __all__ = (
     varReportSinge,
     varReport,
     varGroupsReport,
+    varGroupsPlot,
     binSelector,
+    binAdjuster,
     binFreq,
     binPretty,
     binTree,
@@ -45,6 +48,7 @@ __all__ = (
     hgirdTuner,
     BayesianXGBTuner,
     BayesianLgbmTuner,
+    perfEval,
     shapCheck
 )
 
