@@ -19,6 +19,7 @@ from BDMLtools.base import Base
 from BDMLtools.fun import raw_to_bin_sc,Specials
 from BDMLtools.plotter.base import BaseWoePlotter
 
+
 class EDAReport(Base,TransformerMixin):
     
     """ 
@@ -625,7 +626,6 @@ class varGroupsReport(Base,TransformerMixin,BaseWoePlotter):
         
     """        
     
-    
     def __init__(self,breaks_list_dict,columns,sort_columns=None,target='target',row_limit=0,output_psi=False,psi_base='all',
                  special_values=None,sample_weight=None,b_dtype='float64',
                  n_jobs=-1,verbose=0,out_path=None,tab_suffix='_group'):
@@ -673,8 +673,10 @@ class varGroupsReport(Base,TransformerMixin,BaseWoePlotter):
         out_list=parallel(delayed(self._group_parallel)(X_g_gen,g,self.target,self.columns,
                                                        self.breaks_list_dict,self.row_limit,
                                                        self.special_values,self.b_dtype) for g in X_g_gen.groups)
+        
+        self.report_dict_raw={columns:vtabs for columns,vtabs in out_list}
 
-        report=pd.concat({columns:vtabs for columns,vtabs in out_list},axis=1)
+        report=pd.concat(self.report_dict_raw,axis=1)
         
         report.index.rename([None, 'bin'],inplace=True)
 
@@ -698,45 +700,6 @@ class varGroupsReport(Base,TransformerMixin,BaseWoePlotter):
     def transform(self, X):     
      
         return X
-    
-    
-    def woe_plot_group(self,sort_column=None,figure_size=None,n_jobs=-1,verbose=0):
-                 
-        """ 
-        根据组特征分析报告批量绘图并输出          
-        考虑到组绘图的美观与性能，woe_plot_group只支持一个组水平的绘图，因此varGroupsReport的参数columns长度必须为1，此外row_limit参数必须设定为0
-        Params:
-        ------
-        sort_column=None or list,在绘图中排序组水平,要求组水平值必须与原始数据的组水平一致
-        figure_size=None:matplotlib的画布大小
-        n_jobs=-1:joblib并行任务数量
-        verbose=0:joblib信息打印等级
-        
-        Return:
-        ------
-        fig_out:dict,{特征名:绘图报告figure}
-            
-        """
-        
-        self._check_is_fitted()
-        
-        self._check_param_for_plot(self.row_limit,self.columns)
-
-        fig_out=self._woe_plot_group(self.report_dict['report_all'],sort_column,figure_size,n_jobs,verbose)
-        
-        return fig_out
-    
-    
-    def _check_param_for_plot(self,row_limit,columns):
-        
-        if row_limit != 0:
-            
-            raise ValueError('Param "row_limit" of varGroupsReport instance must set to 0 if call ".woe_plot_group"')
-            
-        if len(columns) != 1:
-            
-            raise ValueError('".woe_plot_group" only support single grouper,reset param "columns" in varGroupsReport instance')    
-        
 
     def _group_parallel(self,X_g_gen,g,target,columns,breaks_list_dict,row_limit,special_values,b_dtype):
     
@@ -949,4 +912,106 @@ class varGroupsReport(Base,TransformerMixin,BaseWoePlotter):
             
         writer.save()     
         print('to_excel done') 
+        
+        
+        
+class varGroupsPlot(Base,BaseWoePlotter):
+    
+    
+    def __init__(self,breaks_list,column,target='target',psi_base='all',sort_column=None,special_values=None,
+                       sample_weight=None,b_dtype='float64',n_jobs=-1,verbose=0):
+        
+        self.breaks_list = breaks_list
+        self.column=column
+        self.target = target
+        self.psi_base=psi_base
+        self.sort_column=sort_column
+        self.special_values=special_values
+        self.sample_weight=sample_weight
+        self.b_dtype=b_dtype
+        self.n_jobs=n_jobs
+        self.verbose=verbose
+        
+        
+    def plot(self,X,figure_size=None):
+        
+        self._check_X(X)
+            
+        fig_out=self._woe_plot_group(X,self.breaks_list,self.target,self.column,self.psi_base,
+                             self.sort_column,figure_size,self.special_values,self.sample_weight,self.b_dtype,
+                             self.n_jobs,self.verbose)  
+        
+        self.fig_out=fig_out
+        
+        return fig_out
+
+    
+    def _woe_plot_group(self,X,breaks_list,target,column,psi_base,sort_column=None,figure_size=None,special_values=None,
+                       sample_weight=None,b_dtype='float64',n_jobs=-1,verbose=0):
+                 
+        """ 
+        根据组特征分析报告批量绘图并输出          
+        考虑到组绘图的美观与性能，woe_plot_group只支持一个组水平的绘图，因此varGroupsReport的参数columns长度必须为1，此外row_limit参数必须设定为0
+        Params:
+        ------
+        X,
+        y,
+        
+        sort_column=None or list,在绘图中排序组水平,要求组水平值必须与原始数据的组水平一致
+        figure_size=None:matplotlib的画布大小
+        n_jobs=-1:joblib并行任务数量
+        verbose=0:joblib信息打印等级
+        
+        Return:
+        ------
+        fig_out:dict,{特征名:绘图报告figure}
+            
+        """
+        
+        self._check_param_for_plot([column])
+        
+        n_jobs=effective_n_jobs(n_jobs)   
+                              
+        p=Parallel(n_jobs=n_jobs,verbose=verbose)
+        
+        res=p(delayed(self._woe_plot_group_single)(X,breaks_list,colname,target,column,psi_base,sort_column,
+                                                   figure_size,special_values,sample_weight,b_dtype) for colname in breaks_list)
+        
+        fig_out={colname:fig for fig,colname in res}             
+
+        return fig_out
+    
+    def _check_param_for_plot(self,columns):
+        
+        # if row_limit != 0:
+            
+        #     raise ValueError('Param "row_limit" of varGroupsReport instance must set to 0 if call ".woe_plot_group"')
+            
+        if len(columns) != 1:
+            
+            raise ValueError('".woe_plot_group" only support single grouper,reset param "columns" in varGroupsReport instance')  
+            
+    def _woe_plot_group_single(self,X,breaks_list,colname,target,column,psi_base='all',sort_column=None,figure_size=None,
+                               special_values=None,sample_weight=None,b_dtype='float64'):
+    
+        varbin_g=varGroupsReport({colname:breaks_list[colname]},
+                       n_jobs=1,target=target,
+                       output_psi=True,
+                       columns=[column],
+                       psi_base=psi_base,
+                       special_values=special_values,
+                       sample_weight=sample_weight,
+                       b_dtype=b_dtype,
+                       row_limit=0,
+                      ).fit(X[[colname]+[column]+[target]]).report_dict_raw
+        
+        binx_g=pd.concat(varbin_g,axis=1).droplevel(0)
+        
+        figure,colname=self._get_plot_single_group(binx_g,sort_column=sort_column,figure_size=figure_size,show_plot=False)
+        
+        return figure,colname        
+        
+        
+        
+        
         
\ No newline at end of file
