@@ -1,5 +1,12 @@
+#!/usr/bin/env python3
+# -*- coding: utf-8 -*-
+"""
+Created on Wed Oct 27 23:08:38 2021
+
+@author: zengke
+"""
 from sklearn.base import TransformerMixin,BaseEstimator
-from sklearn.feature_selection import VarianceThreshold,f_oneway,chi2
+from sklearn.feature_selection import f_classif,chi2
 from category_encoders.ordinal import OrdinalEncoder
 from category_encoders import WOEEncoder
 from joblib import Parallel,delayed,effective_n_jobs
@@ -8,6 +15,7 @@ from sklearn.impute import SimpleImputer
 from BDMLtools.fun import Specials
 from BDMLtools.selector.bin_fun import binFreq
 from sklearn.metrics import roc_auc_score
+from sklearn.preprocessing import StandardScaler
 from pandas.api.types import is_numeric_dtype,is_string_dtype
 import numpy as np
 import pandas as pd
@@ -22,17 +30,19 @@ class prefitModel(Base,BaseEstimator):
     """ 
     预拟合数据，在不进行任何特征工程的前提下，使用全量特征预拟合数据并输出模型指标
     此功能用于在建模之前预估现有取数范围下模型的最终的性能，为取数、y定义的合理性提供参考
+    若X存在object或number以外类型的列，那么这些列将被算法忽略
     
     Parameters:
     ----------
         method='ceiling',预拟合数据方法，可选‘floor’,‘ceiling’
-            floor:地板算法，这里使用线性模型(sklearn对的logit回归(C=0.1))进行prefit  
+            floor:地板算法，这里使用线性模型(sklearn对的logit回归(C=0.1 & solver='saga'))进行prefit  
                  + 分类变量处理方式:进行woe编码
                  + 数值特征缺失值:均值填补,当数据缺失值较多时，建议使用ceiling的lightbgm,其可应对缺失数据
             ceiling:天花板算法,这里使用lightgbm进行prefit，且不进行任何交叉验证
                 + 分类变量处理方式:进行woe编码
-                + 数值特征缺失值:不处理
-        params={'max_depth':3,'learning_rate':0.05,'n_estimators':100},method='ceiling'时lightgbm的参数设定        
+                + 数值特征缺失值:不处理        
+        max_iter:100,logit回归最大迭代次数
+        tree_params={'max_depth':3,'learning_rate':0.05,'n_estimators':100},method='ceiling'时lightgbm的参数设定        
         col_rm=None or list,需要移除的列名list，例如id类
         sample_weight=None or pd.Series,样本权重
         
@@ -42,11 +52,12 @@ class prefitModel(Base,BaseEstimator):
         imputer:sklearn.impute.SimpleImputer object,使用回归时数值特征缺失值处理器
         model:sklearn.linear_model.LogisticRegression or lightgbm.LGBMClassifier object拟合的模型对象
     """      
-    def __init__(self,method='ceiling',params={'max_depth':3,'learning_rate':0.05,'n_estimators':100},
+    def __init__(self,method='ceiling',max_iter=100,tree_params={'max_depth':3,'learning_rate':0.05,'n_estimators':100},
                  col_rm=None,sample_weight=None):
 
         self.method=method
-        self.params=params
+        self.tree_params=tree_params
+        self.max_iter=max_iter
         self.col_rm=col_rm
         self.sample_weight=sample_weight    
         
@@ -61,21 +72,28 @@ class prefitModel(Base,BaseEstimator):
         
         X=X[f_names]
         
-        X_numeric=X.select_dtypes('number');X_categoty=X.select_dtypes('object')
+        X_numeric=X.select_dtypes('number')
+        X_categoty=X.select_dtypes('object')
         
         if self.method=='floor':
             
             X_numeric=pd.DataFrame(self.imputer.transform(X_numeric),
                                    columns=self.imputer.feature_names_in_,
                                    index=X_numeric.index,dtype='float32') 
+            
 
         if self.encoder:
             
-            X_categoty=self.encoder.transform(X_categoty)    
+            X_categoty=self.encoder.transform(X_categoty)
 
         
         X_new=pd.concat([X_numeric,X_categoty],axis=1)[f_names]
         
+        if self.method=='floor':
+            
+            X_new=pd.DataFrame(self._scaler.transform(X_new),columns=self._scaler.feature_names_in_,
+                                       index=X_new.index,dtype='float32')
+        
         return X_new
     
     
@@ -94,11 +112,11 @@ class prefitModel(Base,BaseEstimator):
         
         if self.method=='ceiling':
             
-            self.model=self._fit_lgbm(X,y,self.params,self.sample_weight)
+            self.model=self._fit_lgbm(X,y,self.tree_params,self.sample_weight)
             
         elif self.method=='floor':
             
-            self.model=self._fit_reg(X,y,self.sample_weight)
+            self.model=self._fit_reg(X,y,self.max_iter,self.sample_weight)
             
         else:
             
@@ -131,13 +149,20 @@ class prefitModel(Base,BaseEstimator):
         return lgb
     
     
-    def _fit_reg(self,X,y,sample_weight):
+    def _fit_reg(self,X,y,max_iter,sample_weight):
         
         X_numeric,X_categoty_encode=self._get_X(X,y)
         
         X_new=pd.concat([X_numeric,X_categoty_encode],axis=1)
         
-        logit=LogisticRegression(C=0.1,penalty='l1',solver='saga').fit(X_new,y,sample_weight=sample_weight)  
+        self._scaler=StandardScaler().fit(X_new)
+
+        X_new=pd.DataFrame(self._scaler.transform(X_new),
+                       columns=X_new.columns,
+                       index=X_numeric.index,dtype='float32')
+
+        logit=LogisticRegression(C=0.1,penalty='l2',solver='saga',max_iter=max_iter).fit(X_new,y,
+                                                                                         sample_weight=sample_weight)  
         
         return logit
                 
@@ -162,8 +187,13 @@ class prefitModel(Base,BaseEstimator):
                 
             X_categoty=self.encoder.transform(X_categoty)   
             
+        else:
+            
+            self.encoder=None
+            
             
         return X_numeric,X_categoty
+
     
     
 class fliterByShuffle(Base,BaseEstimator):
@@ -379,16 +409,27 @@ class preSelector(Base,Specials,TransformerMixin):
     
     """ 
     线性预筛选,适用于二分类模型
+    
+    筛选过程(设定为None时代表跳过相应步骤):
+    Step 1.缺失值(所有):缺失率高于用户定义值的列将被筛除
+    Step 2.唯一值(字符)/方差(数值):唯一值占比高于用户定义值列将被筛除/方差低于用户定义值列的列将被筛除
+    Step 3.卡方独立性检验p值(字符)/方差分析p值(数值):p值大于用户定义值的列将被剔除
+    Step 4.乱序筛选(所有):原始顺序与随机顺序后使用模型预测的auc差异小于用户定义值的列将被剔除
+    Step 5.Lightgbm筛选(所有):split重要性低于用户定义值的列将被剔除
+    Step 6.Iv值筛选(所有):等频30箱后iv值低于用户定义值的列将被剔除
+    
     Parameters:
     ----------
         na_pct:float or None,(0,1),默认0.99,缺失率高于na_pct的列将被筛除，设定为None将跳过此步骤
         unique_pct:float or None,(0,1),默认0.99,唯一值占比高于unique_pct的列将被筛除,unique_pct与variance需同时输入，任一设定为None将跳过此步骤
         variance:float or None,默认0,方差低于variance的列将被筛除,将忽略缺失值,unique_pct与variance需同时输入，任一设定为None将跳过此步骤
-        chif_pvalue:float or None,(0,1),默认0.05,大于chif_pvalue的列将被剔除,缺失值将被视为单独一类,为None将跳过此步骤
+        chif_pvalue:float or None,(0,1),默认0.05,大于chif_pvalue的列将被剔除,为None将跳过此步骤
+                    + 卡方计算中，缺失值将被视为单独一类,
+                    + f值计算中，缺失值将被填补为接近+inf和-inf，计算两次，两次结果都不显著的列都将被剔除
         tree_imps:int or None,lightgbm树的split_gain小于等于tree_imps的列将被剔除,默认1，设定为None将跳过此步骤
         tree_size:int,lightgbm树个数,若数据量较大可降低树个数，若tree_imps为None时该参数将被忽略
         auc_limit:float,使用shuffle法计算原始数据与乱序数据的mean_decreasing_auc,小于等于auc_val的特征将被踢出,默认0,建议范围0-0.005
-        s_times:int,shuffle法乱序次数，越多mean_decreasing_auc越具备统计意义，但会增加计算量
+        s_times:int,shuffle法乱序次数，越多的s_times的mean_decreasing_auc越具备统计意义，但这会增加计算量
         iv_limit:float or None使用进行iv快速筛选的iv阈值(数值等频30箱，分类则按照类别分箱)
         out_path:str or None,模型报告路径,将预筛选过程每一步的筛选过程输出到模型报告中
         missing_values:缺失值指代值
@@ -452,51 +493,48 @@ class preSelector(Base,Specials,TransformerMixin):
         self.features_info={}
     
         #开始筛选
-        self.features_info['1.orgin']=X.columns.tolist()
+        self.features_info['0.orgin']=X.columns.tolist()
         
-        keep_col=self.features_info['1.orgin']
+        keep_col=self.features_info['0.orgin']
         
-        print('1.start______________________________________complete')
+        print('0.start__________________________________complete')
         
         #fliter by nan
         if self.na_pct is not None:
             
             keep_col=self._filterByNA(X[keep_col],self.na_pct)
             
-            self.features_info['2.filterbyNA']=keep_col
-
+            self.features_info['1.filterbyNA']=keep_col
+          
+            print('1.filterbyNA_____________________________complete')
             
-            print('2.filterbyNA_____________________________complete')
+        #fliter by unique_pct
+        if self.unique_pct is not None: 
             
+            keep_col=self._filterByUnique(X[keep_col],self.unique_pct)
             
-        #fliter by variance and unique_pct
-        if (self.variance is not None) and (self.unique_pct is not None): 
+            self.features_info['2.filterbyUnique']=keep_col
             
-            keep_col=self._filterByUnique(X[keep_col],self.unique_pct)+self._fliterByVariance(X[keep_col],self.variance)
+            print('2.filterbyUniquepct______________________complete')    
+                      
+        #fliter by variance
+        if self.variance is not None: 
+            
+            keep_col=self._fliterByVariance(X[keep_col],self.variance)
             
             self.features_info['3.filterbyVariance']=keep_col
             
-            print('3.filterbyVariance&Uniquepct_______________________complete')
-
-        
+            print('3.filterbyVariance_______________________complete')
+            
+       
         #fliter by chi and f-value
         if self.chif_pvalue is not None:
             
-            keep_col=self._filterByChisquare(X[keep_col],y,self.chif_pvalue)+self._filterByOneway(X[keep_col],y,self.chif_pvalue)    
+            keep_col=self._filterByChif(X[keep_col],y,self.chif_pvalue)  
             
             self.features_info['4.filterbyChi2Oneway']=keep_col
             
-            print('4.filterbyChi2Oneway_____________________complete')            
-        
-        
-        #fliter by lgbm-tree-imp  
-        if self.tree_imps is not None:
-
-            keep_col=self._filterByTrees(X[keep_col],y,self.tree_size,self.tree_imps)
-            
-            self.features_info['5.filterbyTrees']=keep_col
-            
-            print('5.filterbyTrees__________________________complete')
+            print('4.filterbyChi2Oneway_____________________complete')     
             
             
         #fliter by shuffle  
@@ -504,9 +542,20 @@ class preSelector(Base,Specials,TransformerMixin):
             
             keep_col=fliterByShuffle(s_times=self.s_times,auc_val=self.auc_limit,n_jobs=-1).fit(X[keep_col],y).keep
             
-            self.features_info['6.filterbyShuffle']=keep_col
+            self.features_info['5.filterbyShuffle']=keep_col
 
-            print('6.filterbyShuffle__________________________complete')            
+            print('5.filterbyShuffle________________________complete')  
+        
+        
+        #fliter by lgbm-tree-imp  
+        if self.tree_imps is not None:
+
+            keep_col=self._filterByTrees(X[keep_col],y,self.tree_size,self.tree_imps)
+            
+            self.features_info['6.filterbyTrees']=keep_col
+            
+            print('6.filterbyTrees__________________________complete')
+            
         
         #fliter by iv 
         if self.iv_limit is not None:   
@@ -518,14 +567,14 @@ class preSelector(Base,Specials,TransformerMixin):
             print('7.filterbyIV_____________________________complete')
    
         
-        print('Done_________________________________________________')  
+        print('_____________________________________________Done')  
         
-        #打印筛选汇总信息
+        #summary
         for key in self.features_info:
             
             print('步骤{},保留的特征数:{}'.format(key,len(self.features_info[key])))
         
-        #输出报告    
+        #report
         if self.out_path: 
             
             self.preSelector_report=pd.concat([pd.Series(self.features_info[key],name=key) for key in self.features_info.keys()],axis=1)
@@ -540,101 +589,105 @@ class preSelector(Base,Specials,TransformerMixin):
     
     def _filterByNA(self,X,na_pct):
         
-        """ 
-        缺失值处理
-        """ 
-        
         NAreport=X.isnull().sum().div(len(X))
         
         return NAreport[NAreport<=na_pct].index.tolist() #返回满足缺失率要求的列名
     
     def _filterByUnique(self,X,unique_pct):
-        """ 
-        唯一值处理-分类变量
-        """     
-        X_categoty=X.select_dtypes(include='object')
+  
+        X=X.select_dtypes(include=['object','number'])
+        X_oth=X.select_dtypes(exclude=['object','number'])
         
-        if X_categoty.columns.size:
+        if X.columns.size:
             
-            unique_pct=X_categoty.apply(lambda x:x.value_counts(dropna=False).div(len(X_categoty)).max())    
+            X_unique_pct=X.apply(lambda x:x.value_counts(dropna=False).div(len(X)).max())   
             
-            return unique_pct[unique_pct<unique_pct].index.tolist()
+            return X_unique_pct[X_unique_pct<unique_pct].index.tolist()+X_oth.columns.tolist()
         
         else:
             
-            return []
+            return X_oth.columns.tolist()
+
     
     def _fliterByVariance(self,X,variance):
-        """ 
-        方差处理-连续变量,缺失值将被忽略
-        """     
+
         X_numeric=X.select_dtypes('number')
+        X_oth=X.select_dtypes(exclude='number')
+        
+        #drop constant columns
+        X_numeric=X_numeric.loc[:,X_numeric.apply(lambda col: False if col.unique().size==1 else True)]
         
         if X_numeric.columns.size:
             
-            support_vars=VarianceThreshold(threshold=variance).fit(X_numeric).get_support()    
+            X_var=X_numeric.var(ddof=0)
             
-            return X_numeric.columns[support_vars].tolist()
+            return X_var[X_var>variance].index.tolist()+X_oth.columns.tolist()
         
         else:
             
-            return []
+            return X_oth.columns.tolist()
+        
 
-    def _filterByChisquare(self,X,y,chif_pvalue):
+    def _filterByChif(self,X,y,chif_pvalue):
         
-        """ 
-        特征选择-分类变量:卡方值
-        """
+        #drop constant columns
+        X=X.loc[:,X.apply(lambda col: False if col.unique().size==1 else True)]
         
-        X_categoty=X.select_dtypes(include='object')
         
+        X_categoty=X.select_dtypes('object')
+        
+        X_numeric=X.select_dtypes('number')
+        
+        X_oth=X.select_dtypes(exclude=['object','number'])
+
+        
+        #filter by chi2
         if X_categoty.columns.size:      
             
             X_categoty_encode=OrdinalEncoder().fit_transform(X_categoty.replace(np.nan,'missing'))
             
             p_values=chi2(X_categoty_encode,y)[1]
             
-            return  X_categoty_encode.columns[p_values<chif_pvalue].tolist()#返回满足卡方值要求的列名
+            cate_cols=X_categoty_encode.columns[p_values<chif_pvalue].tolist()
+            
         else:
-            return []
-    
-    def _filterByOneway(self,X,y,chif_pvalue):
-        
-        """ 
-        特征选择-连续变量:方差分析(假定方差齐性)
-        """
-        X_numeric=X.select_dtypes('number')
+            
+            cate_cols=[]
+            
         
+        #filter by oneway
         if X_numeric.columns.size:
             
-            p_values_pos=f_oneway(X_numeric.replace(np.nan,1e10),y)[1]
+            cols_fill=X_numeric.median().to_dict()
             
-            p_values_neg=f_oneway(X_numeric.replace(np.nan,-1e10),y)[1]   
+            p_values_pos=f_classif(X_numeric.fillna(2**31),y)[1]
+            
+            p_values_mean=f_classif(X_numeric.fillna(cols_fill),y)[1]
+            
+            p_values_neg=f_classif(X_numeric.fillna(-2**31),y)[1]             
+            
+            num_cols=X_numeric.columns[(p_values_pos<chif_pvalue) | (p_values_neg<chif_pvalue) | (p_values_mean<chif_pvalue)].tolist()
             
-            return X_numeric.columns[(p_values_pos<chif_pvalue) | (p_values_neg<chif_pvalue)].tolist() #返回满足方差分析的列名
-        
         else:
+                
+            num_cols=[]
             
-            return []
+        
+        return cate_cols+num_cols+X_oth.columns.tolist()
+    
 
     def _filterByTrees(self,X,y,tree_size,tree_imps):
-        """ 
-        特征选择:树模型
-        """
+
         X_numeric=X.select_dtypes(include='number')
-        X_categoty=X.select_dtypes(include='object')
+        X_category=X.select_dtypes(include='object')
         X_oth=X.select_dtypes(exclude=['number','object'])
         
-        if X_oth.columns.size:
-            
-            raise ValueError('dtype only in ("number" or "object")')
         
-        
-        if X_categoty.columns.size:
+        if X_category.columns.size:
             
-            X_categoty_encode=OrdinalEncoder().fit_transform(X_categoty).sub(1)
+            X_category_encode=OrdinalEncoder().fit_transform(X_category).sub(1)
             
-            X_new=pd.concat([X_numeric,X_categoty_encode],axis=1)
+            X_new=pd.concat([X_numeric,X_category_encode],axis=1)
 
             lgb=sLGBMClassifier(
                 boosting_type='gbdt',
@@ -643,11 +696,11 @@ class preSelector(Base,Specials,TransformerMixin):
                 n_estimators=tree_size,
                 subsample=0.7,
                 colsample_bytree=1,
-            ).fit(X_new,y,categorical_feature=X_categoty_encode.columns.tolist())         
+            ).fit(X_new,y,categorical_feature=X_category_encode.columns.tolist())         
 
             lgb_imps=lgb.booster_.feature_importance(importance_type='split')
         
-            return X_new.columns[lgb_imps>tree_imps].tolist()
+            return X_new.columns[lgb_imps>tree_imps].tolist()+X_oth.columns.tolist()
         
         elif X_numeric.columns.size:
             
@@ -662,11 +715,11 @@ class preSelector(Base,Specials,TransformerMixin):
 
             lgb_imps=lgb.booster_.feature_importance(importance_type='split')
         
-            return X_numeric.columns[lgb_imps>tree_imps].tolist()
+            return X_numeric.columns[lgb_imps>tree_imps].tolist()+X_oth.columns.tolist()
         
         else:
             
-            return []
+            return X_oth.columns.tolist()
         
     
     def _filterbyIV(self,X,y,iv_limit):        
@@ -684,32 +737,32 @@ class preSelector(Base,Specials,TransformerMixin):
             
             if not 0<self.na_pct<1:
             
-                raise ValueError("na_pct is float and in (0-1)")
+                raise ValueError("na_pct is float and in (0,1)")
             
         if self.variance is not None:
             
             if not self.variance>=0:
                 
-                raise ValueError("variance is in (0-1)")
+                raise ValueError("variance is in (0,1)")
 
         if self.unique_pct is not None:
             
             if not 0<=self.unique_pct<1:
                 
-                raise ValueError("variance is in (0-1)")
+                raise ValueError("variance is in (0,1)")
                 
                 
         if self.chif_pvalue is not None:
             
             if not 0<self.chif_pvalue<=1:
             
-                raise ValueError("chif_pvalue is float and in (0-1]")
+                raise ValueError("chif_pvalue is float and in (0,1]")
                 
         if self.tree_imps is not None:
             
-            if not self.tree_imps>0:
+            if not self.tree_imps>=0:
                 
-                raise ValueError("tree_imps is in (0,inf]")
+                raise ValueError("tree_imps is in [0,inf]")
                 
         if self.auc_limit is not None:
             
@@ -748,7 +801,4 @@ class preSelector(Base,Specials,TransformerMixin):
         self.preSelector_report.to_excel(writer,sheet_name='3.preSelect')
             
         writer.save()     
-        print('to_excel done') 
-        
-        
-        
+        print('to_excel done') 
\ No newline at end of file
