@@ -14,10 +14,10 @@ from BDMLtools.selector import stepLogit,cardScorer
 from BDMLtools.selector import preSelector,corrSelector,prefitModel
 #from BDMLtools.selector import RFECVSelector
 from BDMLtools.selector import lassoSelector
+from BDMLtools.plotter import  perfEval
 from BDMLtools.encoder import woeTransformer
 from BDMLtools.tuner import girdTuner,hgirdTuner
 from BDMLtools.tuner import BayesianXGBTuner,BayesianLgbmTuner,shapCheck
-
 import scorecardpy as sc
 from sklearn.linear_model import LogisticRegression 
 import pandas as pd
@@ -30,9 +30,13 @@ class test:
         
         self.test_dtStandardization()
         self.test_dtypeAllocator()
-        self.test_binSelector()
+        self.test_outliersTransformer()
+        self.test_nanTransformer()
+        self.test_prefitModel()
+        self.test_preSelector()
         self.test_binSelector()
         self.test_scorecard()
+        self.test_perfEval()
         self.test_tab()
         self.test_tunner()
         
@@ -123,7 +127,7 @@ class test:
                                                           drop_date=False,precision=3).fit(dt)
         dt_1=da.transform(dt)
         
-        dt_1.dtypes
+       
 
         if not np.equal(dt_1[['housing','telephone']].dtypes,['category','category']).all():
             
@@ -205,6 +209,191 @@ class test:
 
         print('dtypeAllocator test successfully')
         
+        
+        
+    def test_outliersTransformer(self):
+        
+        dt=sc.germancredit().copy()
+        da=dtypeAllocator().fit(dt)
+        dt=da.transform(dt)
+        
+        dt_nan=outliersTransformer(columns=['credit.amount'],method='nan').fit_transform(dt)
+        dt_fill=outliersTransformer(columns=['credit.amount'],method='fill').fit_transform(dt)
+        
+        dt['credit.amount'].hist()    
+        
+        dt_nan['credit.amount'].hist()    
+        
+        dt_fill['credit.amount'].hist()
+        
+        if dt_nan['credit.amount'].isnull().sum()!=24:
+            
+            raise ValueError('outliersTransformer error') 
+            
+        if dt_fill['credit.amount'][dt_nan['credit.amount'].isnull()].unique()!=[11792.5]:
+        
+            raise ValueError('outliersTransformer error') 
+            
+        print('outliersTransformer test successfully')
+        
+    
+    def test_nanTransformer(self):
+        
+        dt=pd.DataFrame(
+            {
+                'v1':np.arange(5),
+                'v2':np.array([np.inf,-np.inf,np.nan,5.1,1.0]),
+                'v3':np.array(['g1','','special','g3','nan'])
+            })
+        
+        missing_list=[1,np.nan, -np.inf,np.inf,'nan', '', 'special', 'missing']
+        
+        dt_1=nanTransformer(missing_values=missing_list).fit_transform(dt)
+        
+        if not np.isnan(dt_1.v2[0:2]).all():
+            raise ValueError('nanTransformer error:inf')
+        
+        if not (dt_1.v3.unique()==['g1', 'missing', 'g3']).all():
+            raise ValueError('nanTransformer error:string')
+            
+        if not np.isnan(dt_1['v1'][dt['v1']==1].values).all() :   
+            raise ValueError('nanTransformer error:int')
+            
+        if not np.isnan(dt_1['v2'][dt['v2']==1].values).all() :   
+            raise ValueError('nanTransformer error:float')    
+            
+            
+        missing_dict={
+            'v1':[1,np.nan, -np.inf,np.inf,'nan', '', 'special', 'missing'],
+            'v2':[1,np.nan, -np.inf,np.inf,'nan', '', 'special', 'missing'],
+            'v3':[1,np.nan, -np.inf,np.inf,'nan', '', 'special', 'missing']
+        }    
+                    
+        dt_1=nanTransformer(missing_values=missing_dict,fill_value=(-9999,'na')).fit_transform(dt)
+        
+        if not np.isin(dt_1.v2[0:2],[-9999]).all():
+            raise ValueError('nanTransformer error:inf')
+        
+        if not (dt_1.v3.unique()==['g1', 'na', 'g3']).all():
+            raise ValueError('nanTransformer error:string')
+            
+        if not np.isin(dt_1['v1'][dt['v1']==1].values,[-9999]).all() :   
+            raise ValueError('nanTransformer error:int')
+            
+        if not np.isin(dt_1['v2'][dt['v2']==1].values,[-9999]).all() :   
+            raise ValueError('nanTransformer error:float') 
+            
+        print('nanTransformer test successfully')
+        
+    
+    def test_preSelector(self):
+        
+        X=pd.DataFrame(
+            {
+                'na_test':np.append(np.ones(100),np.repeat(np.nan,900)),#nan pop=0.9
+                'uni_test':np.append(np.repeat('-1',800),np.arange(200)),#-1 unique pop=0.8
+                'var_test':np.repeat(1,1000), #variance = 0
+                'chi2_test':np.repeat('g1',1000), #p_value no significancne
+                'f_test':np.random.randint(0,100,1000), #p_value no significancne      
+            }
+        )
+        
+        y=pd.Series(np.random.randint(0,2,1000),name='target')
+        
+        keep_col=preSelector(na_pct=0.89,
+                   unique_pct=None,
+                   variance=None,
+                   chif_pvalue=None,
+                   tree_imps=None,
+                   auc_limit=None,
+                   iv_limit=None).fit(X[['na_test']],y).keep_col
+
+        if 'na_test' in keep_col:
+            
+            raise ValueError("param na_pct error")
+            
+        keep_col=preSelector(na_pct=None,
+                           unique_pct=0.81,
+                           variance=None,
+                           chif_pvalue=None,
+                           tree_imps=None,
+                           auc_limit=None,
+                           iv_limit=None).fit(X[['uni_test']],y).keep_col
+        
+        if not 'uni_test' in keep_col:
+            
+            raise ValueError("param unique_pct error")
+            
+        keep_col=preSelector(na_pct=None,
+                           unique_pct=None,
+                           variance=0,
+                           chif_pvalue=None,
+                           tree_imps=None,
+                           auc_limit=None,
+                           iv_limit=None).fit(X[['var_test']],y).keep_col
+        
+        if 'var_test' in keep_col:
+            
+            raise ValueError("param variance error")
+            
+        keep_col=preSelector(na_pct=None,
+                           unique_pct=None,
+                           variance=None,
+                           chif_pvalue=0.05,
+                           tree_imps=None,
+                           auc_limit=None,
+                           iv_limit=None).fit(X[['chi2_test','f_test']],y).keep_col
+        
+        if 'chi2_test' in keep_col or 'f_test' in keep_col:
+            
+            raise ValueError("param chif_pvalue error")
+            
+        print('preSelector test successfully')
+        
+
+    def test_prefitModel(self):
+        
+        dt=sc.germancredit().copy()
+        
+        dt['creditability']=dt['creditability'].map({'good':0,'bad':1})
+        
+        dtypes_dict={
+            'num':['age.in.years',
+                 'credit.amount',
+                 'creditability',
+                 'duration.in.month',
+                 'installment.rate.in.percentage.of.disposable.income',
+                 'number.of.existing.credits.at.this.bank',
+                 'number.of.people.being.liable.to.provide.maintenance.for',
+                 'present.residence.since',
+                 ],
+            'str':[
+                   'housing',
+                   'telephone',
+                   'foreign.worker','purpose','job','personal.status.and.sex','property',
+                   'credit.history','savings.account.and.bonds','present.employment.since',
+                   'status.of.existing.checking.account',
+                   'other.installment.plans','other.debtors.or.guarantors']
+        }
+        
+        da=dtypeAllocator(dtypes_dict=dtypes_dict,t_unit='1 D',dtype_num='float64',
+                                            drop_date=True,precision=3).fit(dt)
+        
+        dt_1=da.transform(dt)
+        
+        X=dt_1.drop('creditability',axis=1)
+        y=dt_1['creditability']    
+        
+        from sklearn.metrics import roc_auc_score
+        res=prefitModel(method='floor',max_iter=300,col_rm=['credit.amount','telephone']).fit(X,y)
+        auc=roc_auc_score(y,res.predict_proba(X))
+        print('floor auc:{}'.format(auc)) 
+        
+        res=prefitModel(method='ceiling',tree_params={'max_depth': 2, 'learning_rate': 0.1, 'n_estimators': 50},col_rm=['credit.amount','telephone']).fit(X,y)
+        auc=roc_auc_score(y,res.predict_proba(X))
+        print('ceiling auc:{}'.format(auc)) 
+        
+        print('prefitModel test successfully')
 
 
     def test_binSelector(self):
@@ -246,8 +435,7 @@ class test:
                 print('no monotonic trend shows in {}'.format(key))
                 
         print('binSelector test successfully')
-            
-    
+               
             
     def test_scorecard(self):        
     
@@ -329,8 +517,7 @@ class test:
         
         print("score_equal:{}".format(dt_score_sc['score'].equals(dt_score_bm['score'])))
         
-        print('scorecard test successfully')
-    
+        print('scorecard test successfully')    
     
     
     def test_tab(self):
@@ -394,7 +581,7 @@ class test:
         vtabs_g=varGroupsReport(bin_tree.breaks_list,columns=['client_group'],target=y.name,
                             row_limit=0,output_psi=True,n_jobs=1).fit(X_all)
         
-        figs_g=vtabs_g.woe_plot_group()
+        
         
         print("vtabs test successfully")
     
@@ -455,5 +642,74 @@ class test:
         
         print("tunner test successfully")
         
+        
+    def test_perfEval(self):
+        
+        dt=sc.germancredit().copy()
+        dt['creditability']=dt['creditability'].map({'good':0,'bad':1})
+        
+        da=dtypeAllocator().fit(dt)
+        dt=da.transform(dt)
+        
+        X=dt.drop('creditability',axis=1)
+        y=dt['creditability']
+            
+        breaks_list_user={'age.in.years': [26.0, 30.0, 35.0],
+         'credit.amount': [4000.0, 6200.0, 8000.0],
+         'credit.history': [2.0, 3.0, 4.0],
+         'duration.in.month': [8.0, 16.0, 44.0],
+         'foreign.worker': [1],
+         'housing': [1.0],
+         'installment.rate.in.percentage.of.disposable.income': [2.0, 3.0, 4.0],
+         'job': [2.0, 3.0],
+         'number.of.existing.credits.at.this.bank': [2.0],
+         'number.of.people.being.liable.to.provide.maintenance.for': [2.0],
+         'other.debtors.or.guarantors': [2.0],
+         'other.installment.plans': [2.0],
+         'personal.status.and.sex': [2.0, 3.0],
+         'present.employment.since': [2.0, 3.0],
+         'present.residence.since': [2.0],
+         'property': [1.0, 2.0, 3.0],
+         'purpose': ['retraining%,%car (used)',
+          'radio/television',
+          'furniture/equipment%,%domestic appliances%,%business',
+          'repairs%,%car (new)%,%others%,%education'],
+         'savings.account.and.bonds': [1.0, 2.0, 3.0],
+         'status.of.existing.checking.account': [1.0, 2.0, 3.0],
+         'telephone': [1.0]}
+          
+        bin_bm=varReport(breaks_list_dict=breaks_list_user,n_jobs=1).fit(X,y).var_report_dict
+        
+        dt_woe_bm = woeTransformer(varbin=bin_bm,n_jobs=1).transform(X,y)
+          
+        lr_bm = LogisticRegression(penalty='l1',C=0.9,solver='saga').fit(dt_woe_bm, y)  
+        
+        y_pred=pd.Series(lr_bm.predict_proba(dt_woe_bm)[:,1],index=dt_woe_bm.index)
+        
+        y_true=y
+        
+        group=pd.Series(np.repeat('g-credit data',len(dt_woe_bm)),index=dt_woe_bm.index)
+        
+        sample_weight=pd.Series(np.ones(len(dt_woe_bm)),index=dt_woe_bm.index)
+        
+        figs=perfEval(title='g-credit').plot(y_pred, y_true,group,sample_weight,figure_size=(6,6))
+        
+        card_obj = cardScorer(lr_bm,bin_bm,
+                            odds0=0.05263157894736842,
+                            pdo=50,
+                            points0=600).fit(X)
+        
+        card_bm = card_obj.scorecard
+        
+        dt_score_bm=card_obj.transform(X)
+        
+        figs_score=perfEval(title='g-credit score').plot(dt_score_bm['score'],y_true,group,sample_weight,figure_size=(6,6))
+        
+        print(figs)
+        print(figs_score)
+        
+        print('perfEval test successfully')            
+        
+        
 
 test().test_all()
\ No newline at end of file
